import { constantRoutes, partRoutes, adminRoutes, hospitalRoutes } from '@/router'
import store from '@/store'
import { getUserRolesPermissions } from '@/api/user'
import Vue from 'vue'
/**
 * Use meta.role to determine if the current user has permission
 * @param roles
 * @param route
 */
function hasPermission(roles, route) {
  if (route.meta && route.meta.roles) {
    return roles.some(role => route.meta.roles.includes(role))
  } else {
    return true
  }
}

/**
 * Filter asynchronous routing tables by recursion
 * @param routes asyncRoutes
 * @param roles
 */
export function filterAsyncRoutes(routes, roles) {
  const res = []

  routes.forEach(route => {
    const tmp = { ...route }
    if (hasPermission(roles, tmp)) {
      if (tmp.children) {
        tmp.children = filterAsyncRoutes(tmp.children, roles)
      }
      res.push(tmp)
    }
  })

  return res
}

const state = {
  routes: [],
  addRoutes: []
}

const mutations = {
  SET_ROUTES: (state, routes) => {
    state.addRoutes = routes
    state.routes = constantRoutes.concat(routes)
  }
}
const actions = {
  async generateRoutes({ commit }) {
    return new Promise(resolve => {
      const shopInfo = store.getters.organization // 机构信息
      if (!shopInfo.ui_version_type) {
        shopInfo.ui_version_type = 1
      }
      Vue.prototype.uiVersionType = shopInfo.ui_version_type
      const accessedRoutes = [] // asyncRoutes || []
      let lastRoutes = []
      const userInfo = store.getters.userInfo
      if (userInfo.username === 'superadmin') { // 超级管理员
        const userRoutes = filterRoleRouter2(adminRoutes, shopInfo.ui_version_type)
        lastRoutes = accessedRoutes.concat(userRoutes)
        commit('SET_ROUTES', lastRoutes)
        resolve(lastRoutes)
      } else {
        if (shopInfo.shop_type === '6') { // 为医院
          if (userInfo && userInfo.founder === 1) { // 医院管理
            const userRoutes = filterRoleRouter2(hospitalRoutes, shopInfo.ui_version_type)
            lastRoutes = accessedRoutes.concat(userRoutes)
            commit('SET_ROUTES', lastRoutes)
            resolve(lastRoutes)
          } else {
            getUserRolesPermissions(userInfo.role_id).then(response => {
              const userRoutes = filterRoleRouter(hospitalRoutes, response)
              lastRoutes = accessedRoutes.concat(userRoutes)
              commit('SET_ROUTES', lastRoutes)
              resolve(lastRoutes)
            })
          }
        } else if (shopInfo.shop_type === '5') { // 为科室
          if (userInfo && userInfo.founder === 1) { // 科室管理
            const userRoutes = filterRoleRouter2(partRoutes, shopInfo.ui_version_type)
            lastRoutes = accessedRoutes.concat(userRoutes)
            commit('SET_ROUTES', lastRoutes)
            resolve(lastRoutes)
          } else {
            getUserRolesPermissions(userInfo.role_id).then(response => {
              const userRoutes = filterRoleRouter(partRoutes, response)
              lastRoutes = accessedRoutes.concat(userRoutes)
              commit('SET_ROUTES', lastRoutes)
              resolve(lastRoutes)
            })
          }
        }
      }
    })
  },
  /** 退出时清除 动态挂载的路由 */
  removeRoutes({ commit }) {
    commit('SET_ROUTES', [])
  }
}

/**
 * 递归筛选出有权限的路由
 * @param routers
 * @param names
 * @returns {Array}
 */
function filterRoleRouter(routers, names) {
  const _routers = []
  routers.forEach(item => {
    if (names.includes(item.name) || item.hidden) {
      if (item.children) {
        item.children = filterRoleRouter(item.children, names)
      }
      _routers.push(item)
    }
  })
  return _routers
}

function filterRoleRouter2(routers, type) {
  const _routers = []
  routers.forEach(item => {
    // 筛选出没有 needHidden 或者 needHidden 包含当前机构类型 或者 hidden 为 true 的路由
    if ((!('needShow' in item) || item.needShow === true) &&
      // 检查needHidden是否不存在或不是数组或者不包含当前机构类型
      (!('needHidden' in item) || !Array.isArray(item.needHidden) || !item.needHidden.includes(type))) {
      if (item.children) {
        item.children = filterRoleRouter2(item.children, type)
      }
      _routers.push(item)
    }
  })
  return _routers
}

export default {
  namespaced: true,
  state,
  mutations,
  actions
}
