import Vue from 'vue'
import { login, logout } from '@/api/user'
import * as API_Part from '@/api/calling-part'
import * as Auth from '@/utils/auth'
import router, { resetRouter } from '@/router'
import Storage from '@/utils/storage'
import uuidv1 from 'uuid/v1'
import store from '@/store'
import jwt_decode from 'jwt-decode'

const _user = Storage.getItem('calling_user')
const _shop = Storage.getItem('calling_shop')
const _partid = Storage.getItem('part_id')
const _uid = Storage.getItem('uid')
const uuid = Storage.getItem('calling_uuid')
const state = {
  userInfo: _user ? JSON.parse(_user) : '',
  organization: _shop ? JSON.parse(_shop) : '',
  partId: _partid,
  uuid: uuid
}

const mutations = {

  SET_USERINFO: (state, userInfo) => {
    state.userInfo = userInfo
    Storage.setItem('calling_user', userInfo)
  },
  /**
   * 设置访问令牌
   * @param state
   * @param token
   * @constructor
   */
  SET_ACCESS_TOKEN: (state, token) => {
    const expires = new Date(jwt_decode(token).exp * 1000)
    Storage.setItem('calling_access_token', token, { expires })
  },
  /**
   * 设置刷新令牌
   * @param state
   * @param token
   * @constructor
   */
  SET_REFRESH_TOKEN: (state, token) => {
    const expires = new Date(jwt_decode(token).exp * 1000)
    Storage.setItem('calling_refresh_token', token, { expires })
    // 同时延长用户信息失效时间
    const user = Storage.getItem('calling_user')
    Storage.setItem('calling_user', user, { expires })
  },
  /** 设置机构信息**/

  SET_PARTINFO: (state, partInfo) => {
    state.organization = partInfo
    Storage.setItem('calling_shop', partInfo)
  },

  /**
   * 移除用户信息
   * @param state
   * @constructor
   */
  REMOVE_USER: (state) => {
    state.userInfo = ''
    Storage.removeItem('calling_user')
  },

  REMOVE_PARTINFO: (state) => {
    state.organization = ''
    Storage.removeItem('calling_shop')
  },

  /**
   * 移除访问令牌
   * @param state
   * @constructor
   */
  REMOVE_ACCESS_TOKEN: (state) => {
    Storage.removeItem('calling_access_token')
  },
  /**
   * 移除刷新令牌
   * @param state
   * @constructor
   */
  REMOVE_REFRESH_TOKEN: (state) => {
    Storage.removeItem('calling_refresh_token')
  },
  SET_PARTID: (state, partid) => {
    state.partId = partid
    Storage.setItem('part_id', partid)
  },
  REMOVE_PARTID: (state) => {
    state.partId = ''
    Storage.removeItem('part_id')
  }

}

const actions = {
  // user login
  login({ commit, dispatch }, userInfo) {
    const { username, password } = userInfo
    return new Promise((resolve, reject) => {
      // console.log('后台未做登录，先不登录')
      // commit('SET_PARTID', 1)
      // commit('SET_NAME', username)
      // resolve()
      login({ username: username.trim(), password: password.trim() }).then(res => {
        console.log(res)
        const { access_token, refresh_token, ...user } = res
        const expires = new Date(jwt_decode(refresh_token).exp * 1000)
        if (Date.parse(expires) - Date.parse(new Date()) > 0) {
          console.log('服务器或浏览器时间存在问题，请注意!')
        }
        commit('SET_ACCESS_TOKEN', access_token)
        commit('SET_REFRESH_TOKEN', refresh_token)
        commit('SET_PARTID', res.last_login_shopid)
        // commit('SET_NAME', username)
        commit('SET_USERINFO', { ...user, expires })
        // dispatch('getInfo')
        resolve()
      }).catch(error => {
        reject(error)
      })
    })
  },

  // get user info
  getInfo({ commit, state }) {
    return new Promise((resolve, reject) => {
      if (state.partId > 0) {
        API_Part.get(state.partId).then(response => {
          if (!response) {
            reject('Verification failed, please Login again.')
          }
          // 设置机构是什么模式
          // response.ui_version_type = 1
          // Vue.prototype.uiVersionType = 2
          commit('SET_PARTINFO', response)
          resolve(response)
        }).catch(error => {
          reject(error)
        })
      }
    })
  },

  toggleEditLock({ commit, state }, val) {
    return new Promise((resolve, reject) => {
      commit('SET_EDITLOCK', val)
      resolve()
    })
  },
  setUUID({ commit, state }) {
    return new Promise((resolve, reject) => {
      commit('SET_UUID', uuidv1())
      resolve()
    })
  },
  // user logout
  logout({ commit, state, dispatch }) {
    return new Promise((resolve, reject) => {
      logout().then(res => {
        commit('REMOVE_ACCESS_TOKEN')
        commit('REMOVE_REFRESH_TOKEN')
        commit('REMOVE_USER')
        commit('REMOVE_PARTID')
        commit('REMOVE_PARTINFO')
        resetRouter()

        // reset visited views and cached views
        // to fixed https://github.com/PanJiaChen/vue-element-admin/issues/2485
        dispatch('tagsView/delAllViews', null, { root: true })
        resolve()
      }).catch(err => {
        commit('REMOVE_ACCESS_TOKEN')
        commit('REMOVE_REFRESH_TOKEN')
        commit('REMOVE_USER')
        commit('REMOVE_PARTID')
        commit('REMOVE_PARTINFO')
        reject(err)
      })
    })
  },

  /**
   * 设置访问令牌
   * @param commit
   * @param token
   */
  setAccessTokenAction({ commit }, token) {
    commit('SET_ACCESS_TOKEN', token)
  },
  /**
   * 设置刷新令牌
   * @param commit
   * @param token
   */
  setRefreshTokenAction({ commit }, token) {
    commit('SET_REFRESH_TOKEN', token)
  }

}

export default {
  namespaced: true,
  state,
  mutations,
  actions
}
